use std::collections::{BTreeMap, HashMap};

use indexmap::IndexMap;

use super::{Span, Spanned};

#[derive(Debug, PartialEq, Eq)]
pub struct File<'src> {
    pub decls: IndexMap<Spanned<&'src str>, Expression<'src>>,
}

#[derive(Debug, PartialEq, Eq)]
pub struct Expression<'src> {
    pub expr: Expr<'src>,
    pub span: Span,
}

impl<'src> Expression<'src> {
    pub fn new(expr: Expr<'src>, span: Span) -> Self {
        Self { expr, span }
    }
}

#[derive(Debug, PartialEq, Eq)]
pub enum Expr<'src> {
    Node(
        Spanned<&'src str>,
        Option<Spanned<IndexMap<Spanned<&'src str>, Expression<'src>>>>,
    ),
    SimplePipe(Box<Expression<'src>>, Box<Expression<'src>>),
    NamingPipe(
        Box<Expression<'src>>,
        (Vec<Spanned<&'src str>>, Vec<Spanned<&'src str>>),
        Box<Expression<'src>>,
    ),
    MappingPipe(Box<Expression<'src>>, Box<Expression<'src>>),
    NullPipe(Box<Expression<'src>>, Box<Expression<'src>>),
    MultiPipe(IndexMap<Spanned<&'src str>, Expression<'src>>),
    LetIn(
        IndexMap<Spanned<&'src str>, Box<Expression<'src>>>,
        Box<Expression<'src>>,
    ),
    // $
    Var(&'src str),
    // @
    InputVar(&'src str),
    AttrSet(Spanned<IndexMap<Spanned<&'src str>, Expression<'src>>>),
}
